﻿/// <reference path="../SpeechJS.js" />

/*
* 文件名: AndroidPlayerAdapter.js
* 描述: AndroidPlayer SpeechJS适配器
*
* 功能说明：
* 和Android中的播放器进行交互，让JS可以调用Android来进行放音功能， Android回调JS通知放音进度、音量等
*
* 版本: 1.0.0.0
* 作者: yuwang@iflytek.com
* 日期：2012/1/23
*
* 变更记录：
*
*/

(function (win) {

    function AndroidPlayerAdapter(option) {

        var _this = this;
        _this.id = option.id || "speech-android-player-adapter-" + new Date().getTime();
        
        LogUtil.debug("AndroidPlayerAdapter | enter, id:" + _this.id);
        var _player = new AndroidPlayer(_this.id, option);

        //
        // Method Define 
        //
        this.__proxy__.loadAudio = function (url) {
            /// <summary>加载音频</summary>

            _player.loadAudio(url);
        };

        this.__proxy__.play = function () {
            /// <summary>播放音频</summary>

            _player.play();
        };

        this.__proxy__.playRange = function (beginPos, endPos) {
            /// <summary>播放一段音频</summary>
            /// <param name="beginPos" value="int">开始位置(ms)</param>
            /// <param name="endPos" value="int">结束位置(ms)</param>

            _player.playRange(beginPos, endPos);
        };

        this.__proxy__.pause = function () {
            /// <summary>暂停音频</summary>

            _player.pause();
        };

        this.__proxy__.stop = function () {
            /// <summary>停止音频</summary>

            _player.stop();
        };

        this.__proxy__.getVolume = function () {
            /// <summary>获取音量</summary>

            return _player.getVolume();
        };

        this.__proxy__.setVolume = function (volume) {
            /// <summary>设置音量</summary>
            /// <param name="volume" value="Number">音量值 between 0 and 100</param>

            _player.setVolume(volume);
        };       
        
        this.playWarningTone=function(){
        	_player.playWarningTone();
        }
        
        this.setAudioUri = function(uri){
        	_player.setAudioUri(uri);
        }

        this.__class__.check.call(this);
    }

    var players = [];
    function AndroidPlayer(id, option) {
    	
    	//LogUtil.debug("AndroidPlayer | enter, id:" + id + ", option:" + option);
        for (var i = 0; i < players.length; i++) {
        	//LogUtil.debug("players[" + i+"].id:" + players[i].id);
            if (players[i].id === id) {
            	//LogUtil.debug("recorder has exist");
            	//LogUtil.debug("recorder has exist, player:" + players[i]);
            	//LogUtil.debug("recorder has exist, jsInterface:" + players[i].jsInterface);
                return players[i];
            }
        }

        var player = new AndroidPlayerObj(option);
        player.id = id;
        players.push(player);
        LogUtil.debug("AndroidPlayer create a new player, players length:" + players.length);
        return player;
    }

    function AndroidPlayerObj(option) {
    	var _this = this;
    	_this.events = option.events;
    	_this.androidInterface = window.AndroidPlayerInterface;
    	
    	// java call js
        _this.jsInterface = {
            onReady: function (e) {
                /// <summary>播放器加载完毕</summary>

                var callback = _this.events.onReady;
                //LogUtil.debug("callback:" + callback);
                if (callback) {
                    callback(e);
                }
            },
            onPlayBegin: function (e) {
                /// <summary>开始放音</summary>
                var callback = _this.events.onPlayBegin;
                if (callback) {
                    callback(e);
                }
            },
            onPlaying: function (e) {
                /// <summary>放音中</summary>
                //LogUtil.warn("e.position:" + e.position);
                var callback = _this.events.onPlaying;
                if (callback) {
                    callback(e);
                }
            },
            onStop: function (e) {
                /// <summary>放音停止</summary>
                var callback = _this.events.onStop;
                if (callback) {
                    callback(e);
                }
            },
            onCompletion: function(e){
                /// <summary>放音完成</summary>
                var callback = _this.events.onCompletion;
                if (callback) {
                    callback(e);
                }
            },
            onRangePlayEnd: function(e){
                var callback = _this.events.onRangePlayEnd;
                if (callback) {
                    callback(e);
                } 
            }
            
        };
    };

	// js call java
    AndroidPlayerObj.prototype = {
        loadAudio: function (url) {
            this.androidInterface.loadAudio(url);
        },
        play: function () {
            this.androidInterface.play();
        },
        playRange: function (beginPos, endPos) {
            this.androidInterface.playRange(beginPos, endPos);
        },
        pause: function () {
            this.androidInterface.pause();
        },
        stop: function () {
            this.androidInterface.stop();
        },
        getVolume: function () {
            return this.androidInterface.getVolume();
        },
        setVolume: function (volume) {
            this.androidInterface.setVolume(volume);
        },
        playWarningTone:function(){
            this.androidInterface.playWarningTone();
        },
        setAudioUri:function(uri){
           // this.androidInterface.setAudioUri(uri);
        }
    };

    AndroidPlayerAdapter.prototype = new SpeechJS.PlayerAdapterBase();
    AndroidPlayerAdapter.prototype.constructor = AndroidPlayerAdapter;

    win.AndroidPlayer = AndroidPlayer;
    win.AndroidPlayerAdapter = AndroidPlayerAdapter;

})(window);