﻿/// <reference path="../SpeechJS.js" />

/*
* 文件名: AndroidRecorderAdapter.js
* 描述: AndroidRecorder SpeechJS适配器
*
* 功能说明：
* 和Android中的录音器及引擎进行交互，让JS可以调用Android来进行录音、评测等功能， Android回调JS通知评测结果
*
* 版本: 1.0.0.0
* 作者: yuwang@iflytek.com
* 日期：2012/1/23
*
* 变更记录：
*
*/

(function (win) {
    function AndroidRecorderAdapter(option) {

        var _this = this;
        _this.id = option.id || "speech-recorder-adapter-" + new Date().getTime();

        LogUtil.debug("AndroidRecorderAdapter | enter, id:" + _this.id);
        var _recorder = new AndroidRecorder(_this.id, option);

        this.__proxy__.record = function (recordOprion) {
            /// <summary>开始录音</summary>
            /// <param name="option" type="Object">录音参数</param>
           // LogUtil.debug("AndroidRecorderAdapter record, " + recordOprion);
            return _recorder.record(recordOprion);
        };

        this.__proxy__.stop = function (isForce) {
            /// <summary>停止录音</summary>

            return _recorder.stop(isForce);
        };
        
        this.completeLearn = function(option){
            _recorder.completeLearn(option);
        }
        
        this.learnNext = function(option){
            _recorder.learnNext(option);
        }
        
        this.onTrackSentence = function (option){
            _recorder.onTrackSentence(option);
        }   
        
        /**
         * 开始评测
         */
        this.startEval = function (type, text){
            _recorder.startEval(type, text);
        }
        
        /**
         * 取消评测
         */
        this.cancelEval = function (){
            _recorder.cancelEval();
        }
        
        /**
         * 停止录音，提交评测
         */
        this.stopEval = function(){
            _recorder.stopEval();
        }
              
        this.__class__.check.call(this);
    }

    var recorders = [];
    function AndroidRecorder(id, option) {
        for (var i = 0; i < recorders.length; i++) {
            if (recorders[i].id === id) {
               return recorders[i];
            }
        }

        var recorder = new AndroidRecorderObj(option);
        recorder.id = id;
        recorders.push(recorder);
        
        LogUtil.debug("AndroidRecorder create a new recorder");
        return recorder;
    }

    function AndroidRecorderObj(option) {
    	var _this = this;
    	_this.events = option.events || {};
    	_this.androidInterface = win.AndroidRecorderInterface;
    	LogUtil.debug("AndroidRecorderObj , AndroidRecorderInterface:" + win.AndroidRecorderInterface);
    	_this.jsInterface = {
    		onReady:function(){
        		/// <summary>加载完成</summary>
        		
        		 var callback = _this.events.onReady;
        		 
        		 LogUtil.debug("callback:" + callback);
                 if (callback) {
                     callback();
                 }
        	},
            onRecordBegin: function (e) {
                /// <summary>录音开始事件</summary>
                
            	var callback = _this.events.onRecordBegin;
                if (callback) {
                	callback(e);
                }
            },
            onRecordEnd: function (e) {
                /// <summary>录音结束事件</summary>
                
            	var callback = _this.events.onRecordEnd;
                if (callback) {
                	callback(e);
                }
            },
            onSoundEnergy: function (e) {
                /// <summary>音量回调</summary>
                
            	var callback = _this.events.onSoundEnergy;
                if (callback) {
                	callback(e);
                }
            },
            onTrackInfo: function (e) {
                /// <summary>评测跟踪回调</summary>

            	var callback = _this.events.onTrackInfo;
                if (callback) {
                	callback(e);
                }
            },
            onEvalResult: function (e) {
                /// <summary>评测结果</summary>

            	var callback = _this.events.onEvalResult;
                if (callback) {
                	callback(e);
                }
            },
            onError: function (e) {
                /// <summary>录音出错</summary>

            	var callback = _this.events.onError;
                if (callback) {
                	callback(e);
                }
            },
            onCancelSpeech:function(){
                /// <summary>取消评测</summary>
            	var callback = _this.events.onCancelSpeech;
                if (callback) {
                	callback();
                }
            },
            onCompleteLearn:function(e){
            	var callback = _this.events.onCompleteLearn;
                if (callback) {
                	callback(e);
                }
            }
    	};
    }

    AndroidRecorderObj.prototype = {  		
        record: function (recordOption) {

        },
        stop: function (isForce) {

        },
        completeLearn: function (option) {
            if(option == undefined){
                option = "{}";
            }
            return this.androidInterface.completeLearn(option);
        },
        learnNext: function (option){
            if(option == undefined){
                option = "{}";
            }
            return this.androidInterface.learnNext(option);
        },
        onTrackSentence: function (option){
            if(option == undefined){
                option = "{}";
            }
            return this.androidInterface.onTrackSentence(option);
        },
        startEval:function(type, text){
            return this.androidInterface.startEval(type, text);
        },
        cancelEval:function(){
            return this.androidInterface.cancelEval();
        },
        stopEval: function(){
            return this.androidInterface.stopEval();
        }
    };

    AndroidRecorderAdapter.prototype = new SpeechJS.RecorderAdapterBase();
    AndroidRecorderAdapter.prototype.constructor = AndroidRecorderAdapter;

    win.AndroidRecorder = AndroidRecorder;
    win.AndroidRecorderAdapter = AndroidRecorderAdapter;
})(window);