﻿/// <reference path="SpeechJS.js" />
/// <reference path="const_define.js" />
/// <reference path="jquery-1.10.1.js" />

// 自定义展现器
function TalkShowPresenter(option) {
    var _this = this;

    //当前是否是句子详情
    _this.isDetails = false;
    
    //总成绩，负数代表没有总成绩
    _this.totalScore = -1;
    
    //当前段落索引
    var _currentParagraphIndex = 0;
    
    //合成文本，以段落为单位
    var _synthText = [];
    
    //评测结果
    var _evalResults = [];
    
    /**
    * 根据TTS信息生成Html
    * @Public Method
    */
    _this.__proxy__.buildHTML = function (ttsInfo) {
        return initHTML(ttsInfo);
    };

    /**
    * 段落高亮显示
    * @Public Method
    */
    _this.__proxy__.highlight = function (position) {
    	//如果是当前是句子详情则返回
    	if(_this.isDetails){
    		return
    	}
        var indexInfo = _this.TTS.getIndexByTime(position);
        if (indexInfo.ParagraphIndex > -1 && indexInfo.SentenceIndex > -1
				&& indexInfo.WordIndex > -1) {

            if (!isNeedHighlight(indexInfo)) {
                return;
            }         
            //清除之前语句的播放状态 ,并还原为相应状态    
            restoreItemStatus(_currentParagraphIndex);         
            var paragraph = _this.DOM.Paragraphs[indexInfo.ParagraphIndex];
            _currentParagraphIndex = indexInfo.ParagraphIndex;
            
            if (getItemStatus(paragraph.children[0]) != Const.SelectStatus.onPlaying) {
                // 设置当前句子选中播放状态
        		clearWordStatus(_currentParagraphIndex);
                setItemStatus(paragraph.children[0], Const.SelectStatus.onPlaying);
                gildeToCurrentSen();
            }
        }
    }
	
    /**
    * 清除页面状态
    */
    this.__proxy__.clearHighlight = function () {
    	restoreItemStatus(_currentParagraphIndex);
        _currentParagraphIndex = 0;
        clearScoreStatus();
    };
    
    
    /**
     * 朗读跟踪
     */
    this.__proxy__.handleTrackInfo = function (trackInfo) {
    	//如果是当前是句子详情则返回
    	if(_this.isDetails){
    		return
    	}
    	
        // 句子跟踪
        if (trackInfo.TrackType === 2) {

        }
        // 单词跟踪
        else if (trackInfo.TrackType === 3) {
        	console.log("trackInfo.TrackType="+trackInfo.TrackType);
            
            var thisParagrapgEndIndex = 0;
            for(var i = 0; i<=trackInfo.ParagraphIndex;i++){
            	thisParagrapgEndIndex += _this.DOM.Paragraphs[i].Words.length
            }
            
            if (trackInfo.WordIndex >= thisParagrapgEndIndex
					&& (trackInfo.ParagraphIndex + 1) < _this.DOM.Paragraphs.length) {
            	trackInfo.ParagraphIndex += 1;
            }
            recordSentLine(trackInfo);
        }
    };
    
    this.__class__.check.call(this);
       

    /**
    * 根据TTS信息生成Html
    */
    function initHTML(ttsInfo) {
        var paragraphs = ttsInfo.Paragraphs;
        var html = [];
        var globalIndex = 0;
        var  flag = false;
        // 添加段落
        for (var paragraphIndex = 0, pLength = paragraphs.length; paragraphIndex < pLength; paragraphIndex++) {
            var paragraph = paragraphs[paragraphIndex];
            html.push('<div  class="' + SpeechJS.Defines.ParagraphClass + '" pIndex="' + paragraphIndex + '"><ul class="content_normal"><li>');
            var wordsText = '';
            // 添加句子
            for (var senIndex = 0, sLength = paragraph.Sentences.length; senIndex < sLength; senIndex++) {
                var sen = paragraph.Sentences[senIndex];
                html.push('<span class="' + SpeechJS.Defines.SentenceClass + ' txt_english" sIdx="' + senIndex + '">');
                // 添加单词
                var words = sen.Words;
                for (var wIndex = 0, wLength = words.length; wIndex < wLength; wIndex++) {
                    var word = words[wIndex];
                    //拼接文本，用于引擎评测
                    if (word.PY == "" && word.PT.indexOf("</Font>") > -1) {
                        if(word.PT.indexOf(".") > -1 && senIndex==0){
                            flag = true;
                        }
                        if( flag ){
                            if(word.PT.indexOf(".") > -1 ){
                                html.push('<span wIdx="' + wIndex + '" class="'+cssConst.name_span1 + '" >' + word.PT.replace("</Font>", "") + ' </span>');
                            }else{
                                html.push('<span wIdx="' + wIndex + '" class="'+cssConst.name_span2 + '" >' + word.PT.replace("</Font>", "") + ' </span>');
                            }
                        }else{
                            html.push('<span wIdx="' + wIndex + '" class="'+cssConst.name_span1 + '" >' + word.PT.replace("</Font>", "") + ' </span>');
                        }
                        wordsText  += pushWord(word);
                    } else if (word.PY == "") {
                        html.push('<span class="' + cssConst.symbol_span + '" wIdx="' + wIndex + '">' + word.PT + ' </span>');
                        wordsText  += pushWord(word);
                    }else {
                        html.push('<span class="' + SpeechJS.Defines.WordClass + '" wIdx="' + wIndex + '">' + word.PT + ' </span>');
                        wordsText  += pushWord(word);
                    }
                }
                html.push('</span>');
            }
            _synthText.push(wordsText);
            html.push('<div class="txt_china gray"></div></li></ul></div>');
        }

        return html.join('');
    }

    /**
     * 获取单词
     * @param word
     * @returns {string}
     */
    function pushWord(word){
        var  wordsText='';
        if (word.OT !== '') {
            wordsText = word.OT + ' ';
        }else {
            if(word.PY == "" && word.PT.indexOf("</Font>") > -1)  {
                wordsText = word.PT.replace("</Font>", "") + ' ';
            }else{
                wordsText = word.PT + ' ';
            }
        }
        return wordsText;
    }
    /**
    * 判断当前段落是否可以高亮
    */
    function isNeedHighlight(indexInfo) {
        if (indexInfo.ParagraphIndex === _currentParagraphIndex && _currentParagraphIndex === 0 && getItemStatus(_this.DOM.Paragraphs[_currentParagraphIndex].children[0]) != Const.SelectStatus.onPlaying) {
            return true;
        }
        if (indexInfo.ParagraphIndex != _currentParagraphIndex) {
            return true;
        }
        return false;
    }


    /**
    * 获取段落当前的状态
    * @param itemDivNode
    * @returns
    */
    function getItemStatus(item) {
        switch (item.className) {
            case Const.CssConst.content_normal:
                return Const.SelectStatus.onClear;
                
            case Const.CssConst.content_current:
                return Const.SelectStatus.onPlaying;   
                
            case Const.CssConst.score_very_bad:
            	return Const.SelectStatus.onEvalVeryBad;
            	
            case Const.CssConst.score_bad:
            	return Const.SelectStatus.onEvalBad;
            	
            case Const.CssConst.score_good:
            	return Const.SelectStatus.onEvalGood;
            	
            case Const.CssConst.score_very_good:
            	return Const.SelectStatus.onEvalVeryGood;   
        }
    }

    /**
    * 根据状态为段落设置class
    */
    function setItemStatus(item, state) {
		console.log("state::"+state);
        var className = Const.CssConst.content_normal;
        var isGray = true;
        switch (state) {
            case Const.SelectStatus.onClear:
                className = Const.CssConst.content_normal;
                break;
            case Const.SelectStatus.onPlaying:
                className = Const.CssConst.content_current;
                isGray = false;
                break;
            case Const.SelectStatus.onEvalVeryBad:
                className = Const.CssConst.score_very_bad;
            	break;
            case Const.SelectStatus.onEvalBad:
                className = Const.CssConst.score_bad;
            	break;
            case Const.SelectStatus.onEvalGood:
                className = Const.CssConst.score_good;
            	break;
            case Const.SelectStatus.onEvalVeryGood:
                className = Const.CssConst.score_very_good;
            	break;        
        }
        item.className = className;
		console.log("className::"+className);
    }
    
    /**
     * 段落状态还原
     * @param pIndex
     */
    function restoreItemStatus(pIndex){
    	refreshScoreReport(pIndex);
    }

    /**
    * 滚动条定位至当前语句
    */
    function gildeToCurrentSen() {
        var idx = _currentParagraphIndex;
        if (idx < 0 || idx > _this.DOM.Paragraphs.length || idx === 0) return;
        var sen = _this.DOM.Paragraphs[idx];
        //滚动
        scrollGlide.glide(sen, 200);
    }
    
    /**
     * 获取中文翻译
     */
    function getTranslate(index){
    	/*if(_translate.length > index){
        	return _translate[index];
    	}*/
    	return '';
    }

    /**
     * 获取合成文本
     * index < 0 则返回全部。否则返回当前索引项的文本
     */
    function getSynthText(index){
    	if(index < 0){
    		return _synthText.join('');
    	}
    	else if(_synthText.length > index){
    		return _synthText[index];
    	}
    }
    
    /**
     * 设置录音背景高亮
     * @param indexInfo
     */
    function recordSentLine(trackInfo) {
    	
    	console.log("trackInfo.ParagraphIndex="+trackInfo.ParagraphIndex);
    	
        if (_currentParagraphIndex === trackInfo.ParagraphIndex) {
            return;
        }           
        setItemStatus(_this.DOM.Paragraphs[_currentParagraphIndex].children[0], Const.SelectStatus.onClear);      
        var paragraph = _this.DOM.Paragraphs[trackInfo.ParagraphIndex];
        _currentParagraphIndex = trackInfo.ParagraphIndex;

        if (getItemStatus(paragraph.children[0]) != Const.SelectStatus.onPlaying) {
            // 设置当前句子选中播放状态
            setItemStatus(paragraph.children[0], Const.SelectStatus.onPlaying);
            gildeToCurrentSen();
        }
    }
    
    /**
     * 录音第一句高亮显示
     */
    function firstSenHighlight() {
        _currentParagraphIndex = 0;
        var paragraph = _this.DOM.Paragraphs[_currentParagraphIndex];
        // 设置当前句子为高亮状态
        setItemStatus(paragraph.children[0], Const.SelectStatus.onPlaying);
    }
    
    /**
     * 开始录音
     */
    function recordBegin(){
    	clearScoreStatus();
    	firstSenHighlight();
    }
    
    /**
     * 缓存评测结果到_evalResults
     * 因为评测结果没有段落，所以这里全部以句子的形式存储
     * index 为段落索引
     * index < 0 全篇，否则 单句
     */
    function cacheEvalResult(index, result){
    	if(!result){
    		return;
    	}
    	
    	if(index < 0){
    		_evalResults = result.Sentences;
    	}
    	else{
    		var p = _this.DOM.Paragraphs[index];
    		for(var sIndex = 0, sLength = p.Sentences.length; sIndex < sLength; sIndex ++){
    			var gIndex = p.Sentences[sIndex].GlobalIndex;
        		_evalResults[gIndex] = result.Sentences[sIndex];  			
    		}
    	}
    }
    
    /**
     * 将本地保存的得分信息放入数组
     * @param result
     */
    function setLocalEvalResult(result){
    	if(result){
        	_evalResults = result;
        	showScoreReport();
    	}
    }
     
    /**
     * _evalResults 取值
     * index 为段落索引, -1 返回全部
     */
    function getEvalResult(index){
    	if(index < 0){
    		return _evalResults;
    	}
    	if(_evalResults.length > index){
    		var sens = _this.DOM.Paragraphs[index].Sentences;
    		var sLength = sens.length;
    		if(sLength > 1){
    			var result = [];
        		for(var sIndex = 0; sIndex < sLength; sIndex ++){
        			var gIndex =  sens[sIndex].GlobalIndex;
        			result.push(_evalResults[gIndex]);
        		}
        		return result;
    		}
    		else{
    			var gIndex = sens[0].GlobalIndex;      		
        		return _evalResults[gIndex];   		
    		}

    	}
    	return null;
    }
    
    /**
     * 计算总成绩
     * @returns {Number}
     */
    function computeTotalScore(){
		var score = 0;
    	var result = getEvalResult(-1);
    	if(result && result.length > 0){
    		var rLength = result.length;
    		for(var rIndex = 0; rIndex < rLength; rIndex ++){
    			score += result[rIndex].TotalScore;

    		}
    		score = Math.round(score/rLength);
        	_this.totalScore = score;
        	return score;
    	}
    	return -1;
    }
    
    /**
    *跳转到首部
    */
    function moveToTop(duration){
    	scrollGlide.glideToTop(duration);
    };
    
    /**
     * 显示评测结果
     */
    function showScoreReport(){      
    	for(var pIndex = 0, pLength = _this.DOM.Paragraphs.length; pIndex < pLength; pIndex ++){
    		showParagraphScore(pIndex);
    	}
    }
    
    /**
     * 刷新评测结果
     * pIndex 段落索引
     */
    function refreshScoreReport(pIndex){
		showParagraphScore(pIndex);
    }
    
    /**
     * 显示段落的得分情况，包括笑脸及单词对错
     * @param index
     */
    function showParagraphScore(index){
        var evalResult = getEvalResult(index);
        var smileStatus = Const.SelectStatus.onClear;;
        if(evalResult){
            var totalScore = 0;
            if (isArray(evalResult)) {
                var num = 0;
                for (var rIndex = 0, rLength = evalResult.length; rIndex < rLength; rIndex++) {
                	var sWords = _this.DOM.Paragraphs[index].Sentences[rIndex].Words;
                	var rWords = evalResult[rIndex].Words;
                	//因为引擎返回的结果中单词可能不全，所以这里需要加以判断
                	for(var wIndex = 0, wLength = sWords.length; wIndex < wLength; wIndex ++){
                		var msg = 0;
						var score = 0;
                		if(rWords[wIndex]){
                			msg = rWords[wIndex].DpMsg;
							score = rWords[wIndex].Score;
                		}
                		setWordStatus(sWords[wIndex], msg,score);
                	}
                	
                    totalScore += evalResult[rIndex].TotalScore;
                    num++;
                }
                totalScore = totalScore / num;
            }
            else {
            	var sWords = _this.DOM.Paragraphs[index].Sentences[0].Words;
            	var rWords = evalResult.Words;
            	//因为引擎返回的结果中单词可能不全，所以这里需要加以判断
            	for(var wIndex = 0, wLength = sWords.length; wIndex < wLength; wIndex ++){
            		var msg = 0;
					var score = 0;
            		if(rWords[wIndex]){
            			msg = rWords[wIndex].DpMsg;
						score = rWords[wIndex].Score
            		}
            		setWordStatus(sWords[wIndex], msg,score);
            	}         	
                totalScore = evalResult.TotalScore;
            }
            smileStatus =  getSmileStatus(totalScore);
        }
        setItemStatus(_this.DOM.Paragraphs[index].children[0], smileStatus);				
    }
    
    /**
     * 通过得分获取语句的得分状态(笑脸)
     * @param idx
     * @returns
     */
    function getSmileStatus(score) {
    	score =  Math.round(score);
        score = 100 * (score / 5);
		if (score < 60)
            return Const.SelectStatus.onEvalVeryBad;
        else if (score < 70)
            return Const.SelectStatus.onEvalBad;
        else if (score < 80)
            return Const.SelectStatus.onEvalGood;
        else
            return Const.SelectStatus.onEvalVeryGood;

        return Const.SelectStatus.onClear;
    }
    
    /**
     * 设置单词的对错状态
     * @param code
     */
    function setWordStatus(word, code,score){
    	var status = SpeechJS.Defines.word_miss;
    	switch(code){
    		//正常
	    	case Const.ESEEConst.ISE_RESULT_NO_ERR:
                status += ' ';
                if(score>=4.0){
                    status +=  ' ' +Const.CssConst.word_verygood;
                }else if(score >=2.5 && score <4.0){
                    status +=  ' ' +Const.CssConst.word_good;
                }else{
                    status +=  ' ' +Const.CssConst.word_bad;
                }
	    		break;
        	//漏读
	    	case Const.ESEEConst.ISE_RESULT_MISS:
	    		status = Const.CssConst.word_miss;
	    		break;
        	//增读
	    	case Const.ESEEConst.ISE_RESULT_INSERT:
                status +=  ' ' + Const.CssConst.word_miss;
	    		break;
        	//错读
	    	case Const.ESEEConst.ISE_RESULT_ERR:
	    		status += ' ' +  Const.CssConst.word_bad;
	    		break;
    		default:
                status +=  ' ' + Const.CssConst.word_miss;
    			break;
    	}
    	word.className = status;
    }
    
    /**
     * 清楚单词的颜色值
     */
    function clearWordStatus(pIndex){
    	if(_evalResults.length <= 0){
    		return;
    	}
		var words = _this.DOM.Paragraphs[pIndex].Words;
		for(var wIndex=0, wLength = words.length; wIndex < wLength; wIndex ++){
			words[wIndex].className = '';
		}
    }
    
    /**
     * 清楚页面得分状态，包括笑脸及单词颜色
     */
    function clearScoreStatus(){
    	if(_evalResults.length <= 0){
    		return;
    	}
    	for(var pIndex=0,pLength=_this.DOM.Paragraphs.length; pIndex < pLength; pIndex++){
    		clearWordStatus(pIndex);
    		setItemStatus(_this.DOM.Paragraphs[pIndex].children[0], Const.SelectStatus.onClear);	
    	}
    	_evalResults.length = 0;
    }
    
    /**
     * 判断是否是数组
     */
    function isArray(v) {
        return toString.apply(v) === '[object Array]';
    }
    
    /**************************************************************/
    
    /**
     * 获取中文翻译
     * 暴露给外部使用
     */
    this.getTranslate = getTranslate;
    
    /**
     * 获取合成文本
     * index < 0 则返回全部。否则返回当前索引项的文本
     */
    this.getSynthText = getSynthText;
    
    /**
     * 开始录音
     */
    this.recordBegin = recordBegin;
    
    /**
     * 缓存评测结果
     */
    this.cacheEvalResult = cacheEvalResult;
    
    /**
     * 将本地取出的得分信息放入数组
     */
    this.setLocalEvalResult = setLocalEvalResult;
    
    /**
     * 获取评测结果
     */
    this.getEvalResult = getEvalResult;
    
    /**
     * 显示评测结果
     */
    this.showScoreReport = showScoreReport;
    
    /**
     * 刷新评测结果
     */
    this.refreshScoreReport = refreshScoreReport;
	
	/**
	*跳转到顶部
	*/
	this.moveToTop = moveToTop;
    
}
TalkShowPresenter.prototype = new SpeechJS.PresenterBase();
TalkShowPresenter.prototype.constructor = TalkShowPresenter;