﻿/// <reference path="SpeechJS.js" />
/// <reference path="constDefine.js" />
/// <reference path="jquery-1.10.1.js" />

// 自定义展现器
function TopNewsPresenter(option) {
    var _this = this;
    var _symbols = [];
    var _currentSenIndex = 0;
    //当前是否是句子详情
    _this.isDetails = false;
    //评测结果
    var _evalResults = [];
    //合成文本，以段落为单位
    var _synthText = [];
    
    //总成绩，负数代表没有总成绩
    _this.totalScore = -1;

    this.__proxy__.buildHTML = function (ttsInfo) {
        return initHTML(ttsInfo);
    };

    /**   
    * 句子高亮显示
    * @Public Method
    */
    this.__proxy__.highlight = function (position) {
        //如果是当前是句子详情则返回
        if (_this.isDetails) {
            return
        }
        var indexInfo = _this.TTS.getIndexByTime(position);

        if (indexInfo.ParagraphIndex > -1 && indexInfo.SentenceIndex > -1
				&& indexInfo.WordIndex > -1) {

            if (!isNeedHighlight(indexInfo)) {
                return;
            }

            var sen = _this.DOM.Sentences[indexInfo.SentenceIndex];
            restoreItemStatus(_currentSenIndex);
            _currentSenIndex = indexInfo.SentenceIndex;
            if (getItemStatus(_symbols[sen.GlobalIndex]) != Const.SelectStatus.onPlaying) {
                // 设置当前句子选中播放状态
                setItemStatus(sen, Const.SelectStatus.onPlaying);
                gildeToCurrentSen();
            }
        }
    }

    /**
    * 朗读跟踪
    */
    this.__proxy__.handleTrackInfo = function (trackInfo) {
		
		console.log("trackInfo.TrackType="+trackInfo.TrackType);
		
        //如果是当前是句子详情则返回
        if (_this.isDetails) {
            return
        }
        // 句子跟踪
        if (trackInfo.TrackType === 2) {

        }
        // 单词跟踪
        else if (trackInfo.TrackType === 3) {
            var sen = _this.DOM.Sentences[trackInfo.SentenceIndex];
            var wLen = sen.Words.length;
            if(wLen > 1 && wLen < 10){
            	wLen = wLen - 1;
            }
            else if(wLen < 20){
            	wLen = wLen - 2;
            }
            else{
            	wLen = wLen * 0.9;
            }  
            if (trackInfo.WordIndex >= sen.Words[wLen].GlobalIndex
					&& (trackInfo.SentenceIndex + 1) < _this.DOM.Sentences.length) {
                trackInfo.SentenceIndex += 1;
                LogUtil.info('trackInfo.SentenceIndex::' + trackInfo.SentenceIndex, 'talkshow');
            }
            recordSentLine(trackInfo);
        }

    };

    /**
    * 清除页面状态
    */
    this.__proxy__.clearHighlight = function () {
    	restoreItemStatus(_currentSenIndex);
        _currentSenIndex = 0;
    }

    this.__class__.check.call(this);

    /**
    * 根据TTS信息生成Html
    */
    function initHTML(ttsInfo) {
        var paragraphs = ttsInfo.Paragraphs;
        var html = [];
        var globalIndex = 0;
        // 添加段落
        for (var paragraphIndex = 0, pLength = paragraphs.length; paragraphIndex < pLength; paragraphIndex++) {

            var paragraph = paragraphs[paragraphIndex];
            html.push('<div class="' + SpeechJS.Defines.ParagraphClass + ' txt_english3" pIndex="' + paragraphIndex + '">');
            // 添加句子
            for (var senIndex = 0, sLength = paragraph.Sentences.length; senIndex < sLength; senIndex++) {
                var sen = paragraph.Sentences[senIndex];
                html.push('<span class="' + SpeechJS.Defines.SentenceClass + '" sIdx="' + senIndex + '">');
                var wordsText = '';
                // 添加单词
                var words = sen.Words;
                for (var wIndex = 0, wLength = words.length; wIndex < wLength; wIndex++) {
                    var word = words[wIndex];
                    html.push('<b class="' + SpeechJS.Defines.WordClass + '" wIdx="' + wIndex + '">' + word.PT + ' </b>');
                    //拼接文本，用于引擎评测
                    if (word.OT !== '') {
                        wordsText += word.OT + ' ';
                    }
                    else {
                        wordsText += word.PT + ' ';
                    }
                }

                html.push('</span>');
                _synthText.push(wordsText);

                //添加句子图片符号到页面
                var senImg = $('<div class="point" sIdx="' + senIndex + '" style="position: absolute;"><img src="images/point2.png" /></div>').hide();
                $('#lineSymbol').append(senImg);
                _symbols.push(senImg);
            }

            html.push('</div>');
        }
        return html.join('');
    }

    /**
    * 判断当前段落是否可以高亮
    */
    function isNeedHighlight(indexInfo) {
        if (indexInfo.SentenceIndex === _currentSenIndex && _currentSenIndex === 0 && getItemStatus(_symbols[_currentSenIndex]) != Const.SelectStatus.onPlaying) {
            return true;
        }
        if (indexInfo.SentenceIndex != _currentSenIndex) {
            return true;
        }
        return false;
    }

    /**
    * 设置句子图片符号的位置
    */
    function setLineSymbolPosition() {
        var imgHeight = 19;
        var sen, $firstWord, $lastWord, fTop, lTop, wHeight, sPosition;
        for (var sIndex = 0, sLength = _this.DOM.Sentences.length; sIndex < sLength; sIndex++) {
            sen = _this.DOM.Sentences[sIndex];
            $firstWord = $(sen.Words[0]);
            $lastWord = $(sen.Words[sen.Words.length - 1]);
            fTop = $firstWord.position().top;
            lTop = $lastWord.position().top + $lastWord.height();
            sPosition = fTop + (lTop - fTop) / 2 - imgHeight;
            //var sPosition = $sen.position().top + ($sen.height() / 2) - imgHeight;
            _symbols[sIndex].css("top", sPosition).show();
        }
    }

    /**
    * 获取段落当前的状态
    * @param item _symbols对象
    */
    function getItemStatus(item) {
        switch (item.find('img').attr('src')) {
            case Const.ImgConst.gray_point_img:
                return Const.SelectStatus.onClear;
            case Const.ImgConst.playing_img:
                return Const.SelectStatus.onPlaying;
        }
    }

    /**
    * 根据状态为段落设置class
    * @param item 句子对象
    */
    function setItemStatus(item, state) {
        //默认为WordClass
        var className = SpeechJS.Defines.WordClass;
        var imgSrc;
        switch (state) {
            case Const.SelectStatus.onClear:
                imgSrc = Const.ImgConst.gray_point_img;
                var words = item.Words;
                for (var wIndex = 0, wLength = words.length; wIndex < wLength; wIndex++) {
                    words[wIndex].className = className;
                }
                break;
            case Const.SelectStatus.onPlaying:
                className = ' ' + Const.CssConst.pre_txt;
                imgSrc = Const.ImgConst.playing_img;
                var words = item.Words;
                for (var wIndex = 0, wLength = words.length; wIndex < wLength; wIndex++) {
                    words[wIndex].className = className;
                }
                break;
            case Const.SelectStatus.onEvalVeryBad:
                imgSrc = Const.ImgConst.eval_very_bad_img;
            	break;
            case Const.SelectStatus.onEvalBad:
                imgSrc = Const.ImgConst.eval_bad_img;
            	break;
            case Const.SelectStatus.onEvalGood:
                imgSrc = Const.ImgConst.eval_good_img;
            	break;
            case Const.SelectStatus.onEvalVeryGood:
                imgSrc = Const.ImgConst.eval_very_good_img;
            	break;    
        }

        _symbols[item.GlobalIndex].find('img').attr('src', imgSrc);
    }

    /**
     * 段落状态还原
     * @param pIndex
     */
    function restoreItemStatus(sIndex){
    	refreshScoreReport(sIndex);
    }
    
    /**
    * 滚动条定位至当前语句
    */
    function gildeToCurrentSen() {
        var idx = _currentSenIndex;
        if (idx < 0 || idx > _this.DOM.Sentences.length || idx === 0){
        	return;
        } 

        var words = _this.DOM.Sentences[idx].Words;
        var sen = words[words.length - 1];
        //滚动
        scrollGlide.glide(sen, 200);
    }

    /**
    * 设置录音当前句背景高亮
    * @param indexInfo
    */
    function recordSentLine(trackInfo) {
        if (_currentSenIndex === trackInfo.SentenceIndex) {
            return;
        }
        //清除之前语句的播放状态
        setItemStatus(_this.DOM.Sentences[_currentSenIndex], Const.SelectStatus.onClear);
        var sen = _this.DOM.Sentences[trackInfo.SentenceIndex];
        _currentSenIndex = trackInfo.SentenceIndex
        if (getItemStatus(_symbols[sen.GlobalIndex]) != Const.SelectStatus.onPlaying) {
            // 设置当前句子选中播放状态
            setItemStatus(sen, Const.SelectStatus.onPlaying);
            gildeToCurrentSen();
        }
    }

    /**
    * 获取合成文本
    * index < 0 则返回全部。否则返回当前索引项的文本
    */
    function getSynthText(index) {
        if (index < 0) {
            return _synthText.join('');
        }
        else if (_synthText.length > index) {
            return _synthText[index];
        }
    }

    /**
    * 缓存评测结果到_evalResults
    * index 为句子索引
    * index < 0 全篇，否则 单句
    */
    function cacheEvalResult(index, result) {
        if (!result) {
            return;
        }

        if (index < 0) {
            _evalResults = result.Sentences;
        }
        else {
            _evalResults[index] = result.Sentences[0];
        }
    }
    
    /**
     * 将本地保存的得分信息放入数组
     * @param result
     */
    function setLocalEvalResult(result){
    	if(result){
        	_evalResults = result;
        	showScoreReport();
    	}
    }

    /**
    * _evalResults 取值
    * index 为句子索引
    */
    function getEvalResult(index) {
    	if(index < 0){
    		return _evalResults;
    	}
        if (_evalResults.length > index) {
            return _evalResults[index];
        }
        return null;
    }
    
    /**
     * 计算总成绩
     * @returns {Number}
     */
    function computeTotalScore(){
		var score = 0;
    	var result = getEvalResult(-1);
    	if(result && result.length > 0){
    		var rLength = result.length;
    		for(var rIndex = 0; rIndex < rLength; rIndex ++){
    			score += result[rIndex].TotalScore;

    		}
    		score = Math.round(score/rLength);
        	_this.totalScore = score;
        	return score;
    	}
    	return -1;
    }

    /**
    * 开始录音
    */
    function recordBegin() {
        clearScoreStatus();
        firstSenHighlight();
    }

    /**
    * 录音第一句高亮显示
    */
    function firstSenHighlight() {
        _currentSenIndex = 0;
        var sen = _this.DOM.Sentences[_currentSenIndex];
        // 设置当前句子为高亮状态
        setItemStatus(sen, Const.SelectStatus.onPlaying);
    }
    
    /**
     * 显示评测结果
     */
    function showScoreReport(){      
    	for(var sIndex = 0, sLength = _this.DOM.Sentences.length; sIndex < sLength; sIndex ++){
    		showSentenceScore(sIndex);
    	}
    }
    
    /**
     * 刷新评测结果
     * pIndex 段落索引
     */
    function refreshScoreReport(sIndex){
    	showSentenceScore(sIndex);
    }
    
    /**
     * 显示句子的得分情况，包括笑脸及单词对错
     * @param index
     */
    function showSentenceScore(index){
        var evalResult = getEvalResult(index);
        var smileStatus = Const.SelectStatus.onClear;;
        if(evalResult){
            var totalScore = 0;
        	var sWords = _this.DOM.Sentences[index].Words;
        	var rWords = evalResult.Words;
        	//因为引擎返回的结果中单词可能不全，所以这里需要加以判断
        	for(var wIndex = 0, wLength = sWords.length; wIndex < wLength; wIndex ++){
        		var msg = 16;
                var score = 0;
        		if(rWords[wIndex]){
        			msg = rWords[wIndex].DpMsg;
                    score = rWords[wIndex].Score;
        		}
                setWordStatus(sWords[wIndex], msg,score);
        	}         	
            totalScore = evalResult.TotalScore;
            smileStatus =  getSmileStatus(totalScore);
        }
		setItemStatus(_this.DOM.Sentences[index], smileStatus);		
    }
    
    /**
     * 清楚页面得分状态，包括笑脸及单词颜色
     */
    function clearScoreStatus(){
    	if(_evalResults.length <= 0){
    		return;
    	}
    	for(var sIndex=0,sLength=_this.DOM.Sentences.length; sIndex < sLength; sIndex++){
    		var sentence = _this.DOM.Sentences[sIndex];
    		setItemStatus(sentence, Const.SelectStatus.onClear);
//    		var sWords = sentence.Words;
//    		for(var wIndex = 0, wLength = sWords.length; wIndex < wLength; wIndex ++){
//    			sWords[wIndex].style.color = "#000000"
//    		}
    	}
    	_evalResults.length = 0;
    }
    
    /**
     * 通过得分获取语句的得分状态(笑脸)
     * @param idx
     */
    function getSmileStatus(score) {
    	score =  Math.round(score);
        score = 100 * (score / 5);
		if (score < 60)
            return Const.SelectStatus.onEvalVeryBad;
        else if (score < 70)
            return Const.SelectStatus.onEvalBad;
        else if (score < 80)
            return Const.SelectStatus.onEvalGood;
        else
            return Const.SelectStatus.onEvalVeryGood;

        return Const.SelectStatus.onClear;
    }
    
    /**
     * 设置单词的对错状态
     * @param code
     */
    function setWordStatus(word, code, score){
        var status = SpeechJS.Defines.word_miss;
    	switch(code){
    		//正常
	    	case Const.ESEEConst.ISE_RESULT_NO_ERR:
                status += ' ';
                if(score>=4.0){
                    status += Const.CssConst.score_very_good;
                }else if(score >=2.5 && score <4.0){
                    status += Const.CssConst.score_good;
                }else if(score>=0.0 && score <2.5){
                    status += Const.CssConst.score_bad;
                }
	    		break;
        	//漏读
	    	case Const.ESEEConst.ISE_RESULT_MISS:
	    		status +=  ' ' + Const.CssConst.word_miss;
	    		break;
        	//增读
	    	case Const.ESEEConst.ISE_RESULT_INSERT:
                status +=  ' ' + Const.CssConst.word_miss;
	    		break;
        	//错读
	    	case Const.ESEEConst.ISE_RESULT_ERR:
	    		status += ' ' +  Const.CssConst.word_error;
	    		break;
    		default:
                status +=  ' ' + Const.CssConst.word_miss;
    			break;
    	}
        word.className = status;
    }

    /**
    * 判断是否是数组
    */
    function isArray(v) {
        return toString.apply(v) === '[object Array]';
    }

    /**
    * 设置句子图片符号的位置
    */
    this.setLineSymbolPosition = setLineSymbolPosition;

    /**
    * 开始录音
    */
    this.recordBegin = recordBegin;

    /**
    * 获取合成文本
    * index < 0 则返回全部。否则返回当前索引项的文本
    */
    this.getSynthText = getSynthText;

    /**
    * 缓存评测结果
    */
    this.cacheEvalResult = cacheEvalResult;
    
    /**
     * 将本地取出的得分信息放入数组
     */
    this.setLocalEvalResult = setLocalEvalResult;

    /**
    * 获取评测结果
    */
    this.getEvalResult = getEvalResult;
    
    /**
     * 显示评测结果
     */
    this.showScoreReport = showScoreReport;
    
    /**
     * 刷新评测结果
     */
    this.refreshScoreReport = refreshScoreReport;

}
TopNewsPresenter.prototype = new SpeechJS.PresenterBase();
TopNewsPresenter.prototype.constructor = TopNewsPresenter;